from fastapi import FastAPI, HTTPException, status
from fastapi.responses import StreamingResponse

from .utils import convert_to_bytes, load_database, save_database, generate_data


def create_api(api_keys: {str}, max_size: int, max_data: int, database: str, buffer_size: int):

    api = FastAPI(docs_url=None, redoc_url=None)


    class MaxSizePerRequestError(Exception):
        pass


    class MinSizePerRequestError(Exception):
        pass


    @api.get('/')
    async def test_data(api_key: str, size: str) -> StreamingResponse:
        try:
            if api_key not in api_keys:
                raise HTTPException(
                    status_code=status.HTTP_401_UNAUTHORIZED,
                    detail='Invalid API Key.'
                )

            try:
                size = convert_to_bytes(size)
            except ValueError as err:
                raise HTTPException(
                    status_code=status.HTTP_400_BAD_REQUEST,
                    detail='Invalid format format for size.'
                ) from err

            if size < 0:
                raise MinSizePerRequestError
            elif max_size < size:
                raise MaxSizePerRequestError

            db = load_database(database)
            if max_data <= db['data-used'] + size:
                raise HTTPException(
                    status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                    detail='Service not available.'
                )
            db['data-used'] += size

            save_database(database, db)

            return StreamingResponse(
                status_code=status.HTTP_200_OK,
                content=generate_data(size, buffer_size),
                media_type='application/octet-stream',
                headers={
                    'Content-Length': str(size)
                }
            )

        except MinSizePerRequestError as err:
            raise HTTPException(
                status_code=status.HTTP_416_REQUESTED_RANGE_NOT_SATISFIABLE,
                detail='Size has to be not-negative.'
            ) from err
        except MaxSizePerRequestError as err:
            raise HTTPException(
                status_code=status.HTTP_416_REQUESTED_RANGE_NOT_SATISFIABLE,
                detail=f'Exceeded max size per request of {max_size} Bytes.'
            ) from err

    return api
