import sys
import argparse
import json
import os
from os.path import exists

import uvicorn

from .utils import convert_to_bytes, save_database
from .api import create_api

# Setup Parser
parser = argparse.ArgumentParser()
parser.add_argument('-c', '--config', type=argparse.FileType('r'),
                    default='./config.json', help='Path to config file in JSON format.')


def run(host: str, port: int, api_keys: {str}, max_size: int, max_data: int, database: str, buffer_size: int):
    if not exists(database) or os.stat(database).st_size == 0:
        save_database(database, {'data-used': 0})

    api = create_api(api_keys, max_size, max_data, database, buffer_size)

    uvicorn.run(api, host = host, port = port)


def main():
    args = parser.parse_args(sys.argv[1:])
    config = json.load(args.config)
    host = config['host']
    port = config['port']
    buffer_size = convert_to_bytes(config['buffer-size'])
    max_size = convert_to_bytes(config['max-size'])
    max_data = convert_to_bytes(config['max-data'])
    api_keys = config['keys']
    database = config['database']

    run(host, port, api_keys, max_size, max_data, database, buffer_size)
